<?php
//
// JSON Wizard
// Copyright (c) 2019 Nagios Enterprises, LLC. All rights reserved.
//
// Party
//
// Party hard
//


include_once(dirname(__FILE__) . '/../configwizardhelper.inc.php');

check_json_configwizard_init();

function check_json_configwizard_init()
{
    $name = "check_json";
    $args = array(
        CONFIGWIZARD_NAME => $name,
        CONFIGWIZARD_VERSION => "1.2.1",
        CONFIGWIZARD_TYPE => CONFIGWIZARD_TYPE_MONITORING,
        CONFIGWIZARD_DESCRIPTION => _("Monitor JSON data on web servers."),
        CONFIGWIZARD_DISPLAYTITLE => _("JSON"),
        CONFIGWIZARD_FUNCTION => "check_json_configwizard_func",
        CONFIGWIZARD_PREVIEWIMAGE => "check_json.jpg",
		CONFIGWIZARD_COPYRIGHT => "Copyright &copy; 2018 Nagios Enterprises, LLC.",
		CONFIGWIZARD_AUTHOR => "Nagios Enterprises, LLC",
        CONFIGWIZARD_REQUIRES_VERSION => 550
    );
    register_configwizard($name, $args);
}


/**
 * @param string $mode
 * @param        $inargs
 * @param        $outargs
 * @param        $result
 *
 * @return string
 */
function check_json_configwizard_func($mode = "", $inargs, &$outargs, &$result)
{
    $wizard_name = "check_json";

    // Initialize return code and output
    $result = 0;
    $output = "";

    // Initialize output args - pass back the same data we got
    $outargs[CONFIGWIZARD_PASSBACK_DATA] = $inargs;

    switch ($mode) {
        case CONFIGWIZARD_MODE_GETSTAGE1HTML:
	    $hostname = grab_array_var($inargs, "hostname", "");
            $address = grab_array_var($inargs, "address", "");
	    $port = grab_array_var($inargs, "port", "");
            $address = nagiosccm_replace_user_macros($address);

            $output = '
<h5 class="ul">' . _('Server Information') . '</h5>
<table class="table table-condensed table-no-border table-auto-width">
  <td></tr>To check this host for up/down, the JSON Wizard sets up a TCP check. Most users will want to set this to the same port the host uses to listen for JSON requests. For example, for if we are checking a standard http(s) API that returns JSON, we would use port 80 or 443 here.</td></tr> 
    <tr>
        <td class="vt">
            <label>' . _('Host Name') . ':</label>
        </td>
        <td>
            <input type="text" size="40" name="hostname" id="hostname" value="' . encode_form_val($hostname) . '" class="form-control">
            <div class="subtext">' . _('The host host name that will appear in the CCM.') . '</div>
        </td>
    </tr>
    <tr>
        <td class="vt">
            <label>' . _('Server Address') . ':</label>
        </td>
        <td>
            <input type="text" size="40" name="address" id="address" value="' . encode_form_val($address) . '" class="form-control">
            <div class="subtext">' . _('The IP address or fully qualified DNS name of the server or device you\'d like to monitor services on.') . '</div>
        </td>
    </tr>
    <tr>
        <td class="vt">
            <label>' . _('Server Port') . ':</label>
        </td>
        <td>
            <input type="text" size="5" name="port" id="port" value="' . encode_form_val($port) . '" class="form-control">
            <div class="subtext">' . _('The port of the service.') . '</div>
        </td>
    </tr>
</table>';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE1DATA:

            // Get variables that were passed to us
	    $hostname = grab_array_var($inargs, "hostname", "");
            $address = grab_array_var($inargs, "address", "");
	    $port = grab_array_var($inargs, "port", "");

            // Check for errors
            $errors = 0;
            $errmsg = array();
	    if (($hostname)=="")
		$errmsg[$errors++] = _("No host name specified.");	
            if (($address)=="")
                $errmsg[$errors++] = _("No address specified.");

 	    if (($port) == "")
                $errmsg[$errors++] = _("No port specified.");
			
            if (!is_numeric($port)&&($port !=""))
                $errmsg[$errors++] = _("Invalid port number.");
				
				
            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }

            break;

        case CONFIGWIZARD_MODE_GETSTAGE2HTML:

            // Get variables that were passed to us
	    $hostname = grab_array_var($inargs, "hostname");
            $address = grab_array_var($inargs, "address");
	    $port = grab_array_var($inargs, "port", "");
            //$ha = @gethostbyaddr($address);
            //if ($ha == "")
              //  $ha = $address;
            //$hostname = grab_array_var($inargs, "hostname", $ha);
            $hostname = nagiosccm_replace_user_macros($hostname);

            $services = grab_array_var($inargs, "services", array());
            $services_serial = grab_array_var($inargs, "services_serial");
            if ($services_serial != "") {
                $services = unserialize(base64_decode($services_serial));
            }

            // Number checks
            if (!array_key_exists("number_check", $services))
                $services["number_check"] = array();
            for ($x = 0; $x < 1; $x++) {
                if (!array_key_exists($x, $services["number_check"]))
                    $services["number_check"][$x] = array();
		
		if (!array_key_exists("service_description", $services["number_check"][$x]))
		    $services["number_check"][$x]["service_description"];
                
		if (!array_key_exists("url", $services["number_check"][$x]))
                    $services["number_check"][$x]["url"] = "";
                if (!array_key_exists("key", $services["number_check"][$x]))
                    $services["number_check"][$x]["key"] = "";
                if (!array_key_exists("warning", $services["number_check"][$x]))
                    $services["number_check"][$x]["warning"] = "";
                if (!array_key_exists("critical", $services["number_check"][$x]))
                    $services["number_check"][$x]["critical"] = "";
                if (!array_key_exists("header", $services["number_check"][$x]))
                    $services["number_check"][$x]["header"] = "";
		if (!array_key_exists("return_key", $services["number_check"][$x]))
                    $services["number_check"][$x]["return_key"] = "";
            }
			
	    // String checks
            if (!array_key_exists("string_check", $services))
                $services["string_check"] = array();
            for ($x = 0; $x < 1; $x++) {
                if (!array_key_exists($x, $services["string_check"]))
                    $services["string_check"][$x] = array();

		if (!array_key_exists("service_description", $services["string_check"][$x]))
                    $services["string_check"][$x]["service_description"];

                if (!array_key_exists("url", $services["string_check"][$x]))
                    $services["string_check"][$x]["url"] = "";
                if (!array_key_exists("key", $services["string_check"][$x]))
                    $services["string_check"][$x]["key"] = "";
                if (!array_key_exists("string", $services["string_check"][$x]))
                    $services["string_check"][$x]["string"] = "";
                if (!array_key_exists("result_code_if_match", $services["string_check"][$x]))
                    $services["string_check"][$x]["result_code_if_match"] = "";
                if (!array_key_exists("result_code_if_no_match", $services["string_check"][$x]))
                    $services["string_check"][$x]["result_code_if_no_match"] = "";
		if (!array_key_exists("return_key", $services["string_check"][$x]))
                    $services["string_check"][$x]["return_key"] = "";
            }

            $output = '
<input type="hidden" name="address" value="' . encode_form_val($address) . '">

<h5 class="ul">' . _('Server Details') . '</h5>
<table class="table table-condensed table-no-border table-auto-width">
    <tr>
        <td>
            <label>' . _('Server Address') . ':</label>
        </td>
        <td>
            <input type="text" size="40" name="address" id="address" value="' . encode_form_val($address) . '" class="form-control" disabled>
        </td>
    </tr>
    <tr>
        <td>
            <label>' . _('Host Name') . ':</label>
        </td>
        <td>
            <input type="text" size="20" name="hostname" id="hostname" value="' . encode_form_val($hostname) . '" class="form-control">
            <div class="subtext">' . _('The name you\'d like to have associated with this server or device.') . '</div>
        </td>
    </tr>
</table>

<h5 class="ul">' . _('Numbers') . '</h5>
<p>' . _('Create checks to monitor numbers found in JSON results.') . '</p>
<table class="adddeleterow table table-condensed table-no-border table-auto-width" style="margin-bottom: 10px;">
    <tr>
	<th>' . _('Service Description') . '</th>
        <th>' . _('URL') . '</th>
        <th>' . _('key') . '</th>
        <th>' . _('Warning Threshold') . '</th>
        <th>' . _('Critical Threshold') . '</th>
        <th>' . _('Additional Header(optional)') . '</th>
	<th>' . _('Return Key(optional)') . '</th>
    </tr>';

            for ($x = 0; $x < count($services["number_check"]); $x++) {

                $output .= '<tr>';

				$output .= '<td><input type="text" size="20" name="services[number_check][' . $x . '][service_description]" id="number_check_service_description' . $x . '" value="' . encode_form_val($services["number_check"][$x]["service_description"]) . '" class="form-control"></td>';
				
                $output .= '<td><input type="text" size="45" name="services[number_check][' . $x . '][url]" id="number_check_url' . $x . '" value="' . encode_form_val($services["number_check"][$x]["url"]) . '" class="form-control"></td>';

                $output .= '<td><input type="text" size="20" name="services[number_check][' . $x . '][key]" id="number_check_key' . $x . '" value="' . encode_form_val($services["number_check"][$x]["key"]) . '" class="form-control"></td>';

                $output .= '<td><input type="text" size="10" name="services[number_check][' . $x . '][warning]" id="number_check_warning' . $x . '" value="' . encode_form_val($services["number_check"][$x]["warning"]) . '" class="form-control"></td>';

                $output .= '<td><input type="text" size="10" name="services[number_check][' . $x . '][critical]" id="number_check_critical' . $x . '" value="' . encode_form_val($services["number_check"][$x]["critical"]) . '" class="form-control"></td>';

                $output .= '<td><input type="text" size="30" name="services[number_check][' . $x . '][header]" id="number_check_header' . $x . '" value="' . encode_form_val($services["number_check"][$x]["header"]) . '" class="form-control"></td>';

				$output .= '<td><input type="text" size="30" name="services[number_check][' . $x . '][return_key]" id="number_return_key' . $x . '" value="' . encode_form_val($services["number_check"][$x]["return_key"]) . '" class="form-control"></td>';
				
				$output .= '		<input type="hidden" name="port" value="' . encode_form_val($port) . '">';
				
                $output .= '</tr></table>';

            }

$output .= '<br /><br /><br /><br /><br /><br /><br /><h5 class="ul">' . _('Strings') . '</h5>
<p>' . _('Create checks to monitor strings found in JSON results.') . '</p>
<table class="adddeleterow table table-condensed table-no-border table-auto-width" style="margin-bottom: 10px;">
    <tr>
	<th>' . _('Service Description') . '</th>
        <th>' . _('URL') . '</th>
        <th>' . _('key') . '</th>
        <th>' . _('String') . '</th>
        <th>' . _('Return code if match') . '</th>
        <th>' . _('Return code if no match') . '</th>
	<th>' . _('Additional Header(optional)') . '</th>
	<th>' . _('Return Key(optional)') . '</th>
    </tr>';

            for ($x = 0; $x < count($services["string_check"]); $x++) {

                $output .= '<tr>';

		$output .= '<td><input type="text" size="20" name="services[string_check][' . $x . '][service_description]" id="string_check_service_description' . $x . '" value="' . encode_form_val($services["string_check"][$x]["service_description"]) . '" class="form-control"></td>';
				
                $output .= '<td><input type="text" size="45" name="services[string_check][' . $x . '][url]" id="string_check_url' . $x . '" value="' . encode_form_val($services["string_check"][$x]["url"]) . '" class="form-control"></td>';

                $output .= '<td><input type="text" size="20" name="services[string_check][' . $x . '][key]" id="string_check_key' . $x . '" value="' . encode_form_val($services["string_check"][$x]["key"]) . '" class="form-control"></td>';

				$output .= '<td><input type="text" size="20" name="services[string_check][' . $x . '][string]" id="string_check_string' . $x . '" value="' . encode_form_val($services["string_check"][$x]["string"]) . '" class="form-control"></td>';
				
				
                $output .= '<td><fieldset class="form-control"><select name="services[string_check][' . $x . '][result_code_if_match]" id="string_check_result_code_if_match">
	       <option value = "' . encode_form_val(0) . '">OK</option>
               <option value = "' . encode_form_val(1) . '">WARNING</option>
               <option value = "' . encode_form_val(2) . '">CRITICAL</option>
               <option value = "' . encode_form_val(3) . '">UNKNOWN</option>
		</select>
			</td>';
                
                $output .= '<td><fieldset class="form-control"><select name="services[string_check][' . $x . '][result_code_if_no_match]" id="string_check_result_code_if_no_match">
	       <option value = "' . encode_form_val(0) . '">OK</option>
               <option value = "' . encode_form_val(1) . '">WARNING</option>
               <option value = "' . encode_form_val(2) . '">CRITICAL</option>
               <option value = "' . encode_form_val(3) . '">UNKNOWN</option>
                </select>
			</td>';

				$output .= '<td><input type="text" size="30" name="services[string_check][' . $x . '][header]" id="string_check_header' . $x . '" value="' . encode_form_val($services["string_check"][$x]["header"]) . '" class="form-control"></td>';
				
				$output .= '<td><input type="text" size="30" name="services[string_check][' . $x . '][return_key]" id="string_return_key' . $x . '" value="' . encode_form_val($services["string_check"][$x]["return_key"]) . '" class="form-control"></td>';
				
				$output .= '		<input type="hidden" name="port" value="' . encode_form_val($port) . '">';
				
                $output .= '</tr>';

            }

			
            $output .= '
    </table>
    <div style="height: 20px;"></div>';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE2DATA:

            // get variables that were passed to us
			
            $address = grab_array_var($inargs, "address");
            $hostname = grab_array_var($inargs, "hostname");
            $services = grab_array_var($inargs, "services");
	    $port = grab_array_var($inargs, "port", "");

            // check for errors
	    $errors = 0;
            $number_errors = 0;
	    $string_errors = 0;
            
	    $errmsg = array();
	    $number_errmsg = array();
	    $string_errmsg = array();
	
	    $number_config == NULL;
	    $string_config == NULL;	
		
            if (is_valid_host_name($hostname) == false)
                $errmsg[$errors++] = _("Invalid host name.");
			
	// number_check_
//var_dump($services["number_check"]);
            foreach ($services["number_check"] as $id => $portarr) {

		$service_description = grab_array_var($portarr, "service_description","");
                $url = grab_array_var($portarr, "url", "");
                $key = grab_array_var($portarr, "key", "");
                $warning = grab_array_var($portarr, "warning", "");
		$critical = grab_array_var($portarr, "critical", "");
		$header = grab_array_var($portarr, "header", "");
		$return_key = grab_array_var($portarr, "return_key", "");
				if ( $service_description!="" || $url !="" || $key !="" || $warning!="" || $critical !="" || $header!="" || $return_key="") {
					if ( !$service_description || !$url || !$key || !$warning || !$critical ){
						$number_errmsg[$number_errors++] = _("Invalid number service check configured.\nService:  " .$service_description . " needs all non-optional fields entered.");
					} else {
						$number_config = 1;
					}
				}
            }
			
			
	// string_check_
//var_dump($services["string_check"]);
            foreach ($services["string_check"] as $id => $portarr) {
		$service_description = grab_array_var($portarr, "service_description","");
                $url = grab_array_var($portarr, "url", "");
                $key = grab_array_var($portarr, "key", "");
		$string = grab_array_var($portarr, "string", "");
                $result_code_if_match = grab_array_var($portarr, "result_code_if_match", "");
		$result_code_if_no_match = grab_array_var($portarr, "result_code_if_no_match", "");
		$header = grab_array_var($portarr, "header", "");
		$return_key = grab_array_var($portarr, "return_key", "");
				if ( $service_description!="" || $url!="" || $key!="" || $string!="" ||  $header!="" || $return_key!="") {
					if ( !$service_description || !$url || !$key || !$string ){
						$string_errmsg[$string_errors++] = _("Invalid string service check configured.\nService: " . $service_description . " needs all non-optional fields entered.");
					} else {
						$string_config = 1;
					}
				}
            }
			
		if ($number_config == NULL && $string_config == NULL ) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = "Must configure at least one service check";
                $result = 1;
            }
			
            if ($errors > 0 || $number_errors > 0|| $string_errors > 0 ) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = array_merge($errmsg, $number_errmsg, $string_errmsg);
                $result = 1;
            }
			
            break;


        case CONFIGWIZARD_MODE_GETSTAGE3HTML:

            // get variables that were passed to us
            $address = grab_array_var($inargs, "address");
            $hostname = grab_array_var($inargs, "hostname");
            $services = grab_array_var($inargs, "services");
            $serviceargs = grab_array_var($inargs, "serviceargs");
			$port = grab_array_var($inargs, "port", "");
			
            $services_serial = grab_array_var($inargs, "services_serial", base64_encode(serialize($services)));
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", base64_encode(serialize($serviceargs)));
			
            $output = '
            
        <input type="hidden" name="address" value="' . encode_form_val($address) . '">
        <input type="hidden" name="hostname" value="' . encode_form_val($hostname) . '">
        <input type="hidden" name="services_serial" value="' . $services_serial . '">
        <input type="hidden" name="serviceargs_serial" value="' . $serviceargs_serial . '">
		<input type="hidden" name="port" value="' . encode_form_val($port) . '">
        
        
            ';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE3DATA:

            break;

        case CONFIGWIZARD_MODE_GETFINALSTAGEHTML:

            $output = '
            
            ';
            break;

        case CONFIGWIZARD_MODE_GETOBJECTS:

            $address = grab_array_var($inargs, "address", "");
            $hostname = grab_array_var($inargs, "hostname", "");
			$port = grab_array_var($inargs, "port", "");
			
            $services_serial = grab_array_var($inargs, "services_serial", "");
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");

            $services = unserialize(base64_decode($services_serial));
            $serviceargs = unserialize(base64_decode($serviceargs_serial));

            $hostaddress = $address;

            // save data for later use in re-entrance
            $meta_arr = array();
            $meta_arr["hostname"] = $hostname;
            $meta_arr["address"] = $address;
            $meta_arr["services"] = $services;
            $meta_arr["serivceargs"] = $serviceargs;
			$meta_arr["port"] = $port;
			
            save_configwizard_object_meta($wizard_name, $hostname, "", $meta_arr);

            $objs = array();

            if (!host_exists($hostname)) {
                $objs[] = array(
                    "type" => OBJECTTYPE_HOST,
                    "use" => "xiwizard_check_json_host",
                    "host_name" => $hostname,
                    "address" => $hostaddress,
                    "icon_image" => "check_json.jpg",
                    "statusmap_image" => "check_json.jpg",
					"check_command" => "xiwizard_check_json_host_port_check!" . $port,
                    "_xiwizard" => $wizard_name,
                );
            }

            if (!array_key_exists("number_check", $services))
                $services["number_check"] = array();

            // configure number checks
            foreach ($services["number_check"] as $id => $portarr) {
		
		$service_description = grab_array_var($portarr, "service_description", "");
                $url = grab_array_var($portarr, "url", "");
                $key = grab_array_var($portarr, "key", "");
                $warning = grab_array_var($portarr, "warning", "");
                $critical = grab_array_var($portarr, "critical", "");
                $header = grab_array_var($portarr, "header", "");
				$return_key = grab_array_var($portarr, "return_key", "");

				if($return_key==null){
					$return_key=$key;
				}
				
                //echo "PROCESSING: $id -> ".serialize($portarr)."<BR>\n";

				if ( have_value($url) ) {
               			    $svc_description = $service_description;

				    $use = "xiwizard_check_json_service";
                		    $check_command = "check_json_number!" . $url . "!" . $key . "!" . $warning . "!" . $critical . "!" . $header . "!" . $return_key;
				
                		    $objs[] = array(
                    			"type" => OBJECTTYPE_SERVICE,
                    			"host_name" => $hostname,
                    			"service_description" => $svc_description,
                    			"use" => $use,
                    			"check_command" => $check_command,
                    			"_xiwizard" => $wizard_name,
                		    );
				}
			}
			
			
			// configure string checks

            foreach ($services["string_check"] as $id => $portarr) {
		
		$service_description = grab_array_var($portarr, "service_description", "");	
                $url = grab_array_var($portarr, "url", "");
                $key = grab_array_var($portarr, "key", "");
                $string = grab_array_var($portarr, "string", "");
                $result_code_if_match = grab_array_var($portarr, "result_code_if_match", "");
                $result_code_if_no_match = grab_array_var($portarr, "result_code_if_no_match", "");
				$header = grab_array_var($portarr, "header", "");
				$return_key = grab_array_var($portarr, "return_key", "");
				
				if($return_key==null){
					$return_key=$key;
				}
				
                //echo "PROCESSING: $id -> ".serialize($portarr)."<BR>\n";

				if ( have_value($url) ) {
                		    $svc_description = $service_description;

				    $use = "xiwizard_check_json_service";
                 		    $check_command = "check_json_string!" . $url . "!" . $key . "!" . $string . "!" . $result_code_if_match . "!" . $result_code_if_no_match . "!" . $header . "!" . $return_key;
				
                		    $objs[] = array(
                    			"type" => OBJECTTYPE_SERVICE,
                    			"host_name" => $hostname,
                    			"service_description" => $svc_description,
                    			"use" => $use,
                    			"check_command" => $check_command,
                    			"_xiwizard" => $wizard_name,
                		    );
				}
            }

			
            // return the object definitions to the wizard
            $outargs[CONFIGWIZARD_NAGIOS_OBJECTS] = $objs;

            break;

        default:
            break;
    }

    return $output;
}

